package database

import (
	"fmt"
	"os"
	"sync"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

var dbInstance *gorm.DB

func initialize() {
	var err error

	host := os.Getenv("POSTGRES_HOST")
	user := os.Getenv("POSTGRES_USER")
	pass := os.Getenv("POSTGRES_PASSWORD")
	name := os.Getenv("POSTGRES_DB")
	port := os.Getenv("POSTGRES_PORT")
	sslMode := os.Getenv("SSL_MODE")

	print(host)
	dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%s sslmode=%s", host, user, pass, name, port, sslMode)
	print(dsn)

	dbInstance, err = gorm.Open(postgres.Open(dsn), &gorm.Config{})
	if err != nil {
		panic("failed to connect database")
	}

	dbInstance.AutoMigrate(&models.Pengguna{})
	dbInstance.AutoMigrate(&models.Topik{})
	dbInstance.AutoMigrate(&models.Pendaftaran{})
	dbInstance.AutoMigrate(&models.UsulanJudul{})

	dbInstance.AutoMigrate(&models.SystemLogs{})
	dbInstance.AutoMigrate(&models.BimbinganLogs{})
	dbInstance.AutoMigrate(&models.Berkas{})
	dbInstance.AutoMigrate(&models.Topik{})
	dbInstance.AutoMigrate(&models.BatchSeminar{})
	dbInstance.AutoMigrate(&models.BatchSidang{})
	dbInstance.AutoMigrate(&models.JadwalSidang{})
	dbInstance.AutoMigrate(&models.JadwalSeminar{})

	dbInstance.AutoMigrate(&models.Pendaftaran{})
	dbInstance.AutoMigrate(&models.PeriodPendaftaran{})
	dbInstance.AutoMigrate(&models.SchedulingSidang{})
	dbInstance.AutoMigrate(&models.SchedulingSeminar{})
}

func GetInstance() *gorm.DB {
	lock := &sync.Mutex{}

	if dbInstance == nil {
		lock.Lock()
		defer lock.Unlock()

		if dbInstance == nil {
			initialize()
		}
	}

	return dbInstance
}
