package usecase

import (
	"strconv"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/entity"
)

type BatchSidangUsecase interface {
	AddBatchSidang(bimbinganLog entity.BatchSidang) (entity.BatchSidang, error)
	GetBatchSidang() ([]entity.BatchSidang, error)
	DeleteBatchSidang(id string) (entity.BatchSidang, error)
	GetBatchSidangCurrent() ([]entity.BatchSidang, error)
	SetLiveBatch(id string) (entity.BatchSidang, error)
	GetBatchSidangLive() ([]entity.BatchSidang, error)
}

type BatchSidangUc struct {
	batchSidangRepo BatchSidangRepository
}

func NewBatchSidangUseCase(batchSidangRepo BatchSidangRepository) *BatchSidangUc {
	return &BatchSidangUc{
		batchSidangRepo: batchSidangRepo,
	}
}

func (uc *BatchSidangUc) AddBatchSidang(batchSidang entity.BatchSidang) (entity.BatchSidang, error) {
	startDate, _ := time.Parse("2006-01-02T15:04:05Z", batchSidang.StartDate)
	month := startDate.Month()
	year := startDate.Year()

	if month <= 6 {
		batchSidang.PeriodSemester = "II"
		batchSidang.PeriodYear = strconv.Itoa(year-1) + "/" + strconv.Itoa(year)
	} else {
		batchSidang.PeriodSemester = "I"
		batchSidang.PeriodYear = strconv.Itoa(year) + "/" + strconv.Itoa(year+1)
	}

	newBatchSidang, err := uc.batchSidangRepo.NewBatchSidang(batchSidang)

	if err != nil {
		return newBatchSidang, err
	}

	return newBatchSidang, nil

}

func (uc *BatchSidangUc) GetBatchSidang() ([]entity.BatchSidang, error) {
	return uc.batchSidangRepo.GetSidangBatches()
}

func (uc *BatchSidangUc) DeleteBatchSidang(id string) (entity.BatchSidang, error) {
	return uc.batchSidangRepo.DeleteBatch(id)
}

func (uc *BatchSidangUc) GetBatchSidangCurrent() ([]entity.BatchSidang, error) {
	return uc.batchSidangRepo.GetBatchSidangCurrent()
}

func (uc *BatchSidangUc) SetLiveBatch(id string) (entity.BatchSidang, error) {
	return uc.batchSidangRepo.SetLiveBatch(id)
}

func (uc *BatchSidangUc) GetBatchSidangLive() ([]entity.BatchSidang, error) {
	return uc.batchSidangRepo.GetBatchSidangLive()
}
