package repository

import (
	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/entity"
	"gorm.io/gorm"
)

type BimbinganLogRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewBimbinganLogRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *BimbinganLogRepo {
	return &BimbinganLogRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

//TODO UPDATE BIMBINGAN LOG STATUS BY ID

func (repo *BimbinganLogRepo) NewBimbinganLog(berkas []entity.Berkas, bimbinganLog entity.BimbinganLog) (output entity.BimbinganLogBerkas, err error) {
	bimbinganLog.ID = uuid.New().String()
	berkasRepo := NewBerkasRepository(repo.DBWrite, repo.DBRead)
	berkas, err = berkasRepo.NewBatchBerkas(bimbinganLog.ID, berkas)
	if err != nil {
		return
	}

	result := repo.DBWrite.Create(&bimbinganLog)
	if result.Error != nil {
		return entity.BimbinganLogBerkas{}, result.Error
	}

	bimbingaLogBerkas := entity.BimbinganLogBerkas{
		ID:              bimbinganLog.ID,
		IDMahasiswa:     bimbinganLog.IDMahasiswa,
		Date:            bimbinganLog.Date,
		LaporanKemajuan: bimbinganLog.LaporanKemajuan,
		Todo:            bimbinganLog.Todo,
		NextBimbingan:   bimbinganLog.NextBimbingan,
		Status:          bimbinganLog.Status,
		Berkas:          berkas,
	}
	return bimbingaLogBerkas, nil
}

func (repo *BimbinganLogRepo) GetBimbinganLogByStudentId(limit int, offset int, idMahasiswa string) (output []entity.BimbinganLogBerkas, err error) {
	var bimbinganLogs []entity.BimbinganLog
	var bimbinganLogsBerkas []entity.BimbinganLogBerkas

	result := repo.DBRead.Where("id_mahasiswa = ?", idMahasiswa).Limit(limit).Offset(offset).Find(&bimbinganLogs)
	if result.Error != nil {
		return nil, err
	}

	berkasRepo := NewBerkasRepository(repo.DBWrite, repo.DBRead)

	for _, log := range bimbinganLogs {
		var logBerkas entity.BimbinganLogBerkas
		logBerkas.ID = log.ID
		logBerkas.IDMahasiswa = log.IDMahasiswa
		logBerkas.Date = log.Date
		logBerkas.LaporanKemajuan = log.LaporanKemajuan
		logBerkas.Todo = log.Todo
		logBerkas.NextBimbingan = log.NextBimbingan
		logBerkas.Status = log.Status

		berkas, err := berkasRepo.GetBerkasByLogID(log.ID)
		if err != nil {
			return nil, err
		}
		logBerkas.Berkas = berkas

		bimbinganLogsBerkas = append(bimbinganLogsBerkas, logBerkas)
	}

	return bimbinganLogsBerkas, nil
}

func (repo *BimbinganLogRepo) UpdateBimbinganLogStatus(logID string, newStatus bool) error {
	updateStatus := map[string]interface{}{
		"status": newStatus,
	}

	result := repo.DBWrite.Model(&entity.BimbinganLog{}).Where("id = ?", logID).Updates(updateStatus)
	if result.Error != nil {
		return result.Error
	}

	if result.RowsAffected == 0 {
		return gorm.ErrRecordNotFound
	}

	return nil
}
