package usecase

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type BimbinganLogUsecase interface {
	AddBimbinganLog(bimbinganLog entity.BimbinganLogBerkas) (entity.BimbinganLogBerkas, error)
	GetBimbinganLog(param utils.LimitOffset, idMahasiswa string) ([]entity.BimbinganLogBerkas, error)
	UpdateBimbinganLogStatus(id string, status bool) error
}

type BimbinganLogUc struct {
	bimbinganLogRepo BimbinganLogRepository
}

func NewBimbinganLogUseCase(bimbinganLogRepo BimbinganLogRepository) *BimbinganLogUc {
	return &BimbinganLogUc{
		bimbinganLogRepo: bimbinganLogRepo,
	}
}

func (uc *BimbinganLogUc) AddBimbinganLog(bimbinganLogBerkas entity.BimbinganLogBerkas) (entity.BimbinganLogBerkas, error) {
	berkas := bimbinganLogBerkas.Berkas
	bimbinganLog := entity.BimbinganLog{
		IDMahasiswa:     bimbinganLogBerkas.IDMahasiswa,
		Date:            bimbinganLogBerkas.Date,
		LaporanKemajuan: bimbinganLogBerkas.LaporanKemajuan,
		Todo:            bimbinganLogBerkas.Todo,
		NextBimbingan:   bimbinganLogBerkas.NextBimbingan,
	}

	newBimbinganLogBerkas, err := uc.bimbinganLogRepo.NewBimbinganLog(berkas, bimbinganLog)

	if err != nil {
		return newBimbinganLogBerkas, err
	}

	return newBimbinganLogBerkas, nil

}

func (uc *BimbinganLogUc) GetBimbinganLog(param utils.LimitOffset, idMahasiswa string) ([]entity.BimbinganLogBerkas, error) {
	return uc.bimbinganLogRepo.GetBimbinganLogByStudentId(param.Limit, param.Offset, idMahasiswa)
}

func (uc *BimbinganLogUc) UpdateBimbinganLogStatus(id string, status bool) error {
	return uc.bimbinganLogRepo.UpdateBimbinganLogStatus(id, status)
}
