package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminBimbinganLogHandler struct {
	bimbinganLogUsecase usecase.BimbinganLogUsecase
}

func NewAdminBimginganLogHandler(cfg config.BimbinganLogTransportConfig) *AdminBimbinganLogHandler {
	bimbinganLogRepository := repository.NewBimbinganLogRepository(cfg.DBWrite, cfg.DBRead)
	bimbinganLogUsecase := usecase.NewBimbinganLogUseCase(bimbinganLogRepository)

	return &AdminBimbinganLogHandler{
		bimbinganLogUsecase: bimbinganLogUsecase,
	}

}

func (t *AdminBimbinganLogHandler) MountAdmin(group *echo.Group) {
	group.GET("/bimbingan-logs", t.GetBimbinganLog)
}

func (t *AdminBimbinganLogHandler) GetBimbinganLog(c echo.Context) error {
	param, err := utils.GetLimitOffset(c.QueryParams())
	idMahasiswa := c.QueryParam("id_mahasiswa")
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}
	listBimbinganLog, err := t.bimbinganLogUsecase.GetBimbinganLog(param, idMahasiswa)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Bimbingan logs successfully retreived", listBimbinganLog))
}
