package repository

import (
	"errors"
	"fmt"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/entity"
	"gorm.io/gorm"
)

type DashboardDosbingRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardDosbingRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardDosbingRepo {
	return &DashboardDosbingRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardDosbingRepo) GetMahasiswaByDosenId(idDosen string, offset, size int) (output []entity.MahasiswaBimbingan, err error) {
	var mahasiswaBimbingan []entity.MahasiswaBimbingan

	var dosen models.Pengguna
	var mahasiswa models.Pengguna
	var topik models.Topik
	var pendaftaran []models.Pendaftaran
	var logBimbingan models.BimbinganLogs
	var statusBimbingan string

	result := repo.DBRead.Where("id = ?", idDosen).First(&dosen)
	if result.Error != nil {
		return nil, errors.New("dosen tidak ditemukan")
	}

	result = repo.DBRead.Where("id_dosen = ? AND status = TRUE", idDosen).Limit(size).Offset(offset).Find(&pendaftaran)
	if result.Error != nil {
		return nil, errors.New("tidak ada mahasiswa bimbingan")
	}

	for _, pendaftar := range pendaftaran {

		repo.DBRead.Where("id = ?", pendaftar.IdMahasiswa).First(&mahasiswa)
		repo.DBRead.Where("id = ?", pendaftar.IdTopik).First(&topik)
		result = repo.DBRead.Where("id_mahasiswa = ?", pendaftar.IdMahasiswa).Order("date desc").First(&logBimbingan)

		if result.Error != nil {
			statusBimbingan = "TERKENDALA"
		} else {
			currentDate := time.Now()
			diff := currentDate.Sub(logBimbingan.Date).Hours() / 24
			if diff <= 30 {
				statusBimbingan = "LANCAR"
			} else if diff > 30 && diff < 90 {
				statusBimbingan = "BUTUH BIMBINGAN"
			} else if diff >= 90 {
				statusBimbingan = "TERKENDALA"
			}
		}

		topik := entity.TopikMahasiswaBingan{
			Judul: topik.Judul,
			Id:    topik.Id,
		}

		bimbingan := entity.MahasiswaBimbingan{
			IdMahasiswa: mahasiswa.Id,
			Nama:        mahasiswa.Nama,
			NIM:         mahasiswa.NIM,
			Topik:       topik,
			Status:      statusBimbingan,
		}

		mahasiswaBimbingan = append(mahasiswaBimbingan, bimbingan)
	}

	return mahasiswaBimbingan, nil

}

func (repo *DashboardDosbingRepo) GetAmountMahasiswaByDosenId(idDosen string) (output []entity.JumlahMahasiswa, err error) {
	var jumlahMahasiswa entity.JumlahMahasiswa

	var dosen models.Pengguna
	var mahasiswaIF int64
	var mahasiswaSTI int64

	result := repo.DBRead.Where("id = ?", idDosen).First(&dosen)
	if result.Error != nil {
		return nil, errors.New("dosen tidak ditemukan")
	}

	// Mahasiswa IF
	result = repo.DBRead.Model(&models.Pendaftaran{}).
		Joins("JOIN pengguna m ON pendaftarans.id_mahasiswa = m.id").
		Where("pendaftarans.id_dosen = ? AND m.nim LIKE ?", idDosen, "135%").
		Count(&mahasiswaIF)
	// result = repo.DBRead.Where("id = ? AND NIM LIKE ?", dosen.Id, "135%").Find(&mahasiswa)
	if result.Error != nil {
		fmt.Println(result)
		mahasiswaIF = 0
	}
	jumlahMahasiswa.MahasiswaIF = int(mahasiswaIF)

	// Mahasiswa STI
	result = repo.DBRead.Model(&models.Pendaftaran{}).
		Joins("JOIN pengguna m ON pendaftarans.id_mahasiswa = m.id").
		Where("pendaftarans.id_dosen = ? AND m.nim LIKE ?", idDosen, "182%").
		Count(&mahasiswaSTI)
	// result = repo.DBRead.Where("id_dosen = ? AND NIM LIKE ?", dosen.Id, "182%").Find(&mahasiswa)
	if result.Error != nil {
		fmt.Println(result)
		mahasiswaSTI = 0
	}
	jumlahMahasiswa.MahasiswaSTI = int(mahasiswaSTI)

	return []entity.JumlahMahasiswa{jumlahMahasiswa}, nil

}

func (repo *DashboardDosbingRepo) GetAmountStatusBimbinganMahasiswaByDosenId(idDosen string) (output []entity.StatusBimbinganMahasiswa, err error) {
	var statusBimbinganMahasiswa entity.StatusBimbinganMahasiswa

	var dosen models.Pengguna
	var mahasiswa models.Pengguna
	var pendaftaran []models.Pendaftaran
	var logBimbingan models.BimbinganLogs

	result := repo.DBRead.Where("id = ?", idDosen).First(&dosen)
	if result.Error != nil {
		return nil, errors.New("dosen tidak ditemukan")
	}

	result = repo.DBRead.Where("id_dosen = ? AND status = TRUE", idDosen).Find(&pendaftaran)
	if result.Error != nil {
		return nil, errors.New("tidak ada mahasiswa bimbingan")
	}

	for _, pendaftar := range pendaftaran {

		repo.DBRead.Where("id = ?", pendaftar.IdMahasiswa).First(&mahasiswa)
		result = repo.DBRead.Where("id_mahasiswa = ?", pendaftar.IdMahasiswa).Order("date desc").First(&logBimbingan)

		if result.Error != nil {
			statusBimbinganMahasiswa.Terkendala++
		} else {
			currentDate := time.Now()
			diff := currentDate.Sub(logBimbingan.Date).Hours() / 24
			if diff <= 30 {
				statusBimbinganMahasiswa.Lancar++
			} else if diff > 30 && diff < 90 {
				statusBimbinganMahasiswa.ButuhBimbingan++
			} else if diff >= 90 {
				statusBimbinganMahasiswa.Terkendala++
			}
		}
	}

	return []entity.StatusBimbinganMahasiswa{statusBimbinganMahasiswa}, nil
}
