package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_ta/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_ta/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_ta/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminDashboardTimHandler struct {
	dashboardTimUseCase usecase.DashboardTimUseCase
}

func NewAdminDashboardTimHandler(cfg config.DashboardTimTransportConfig) *AdminDashboardTimHandler {
	dashboardTimRepository := repository.NewDashboardTimRepository(cfg.DBWrite, cfg.DBRead)
	dashboardTimUsecase := usecase.NewDashboardTimUsecase(dashboardTimRepository)

	return &AdminDashboardTimHandler{
		dashboardTimUseCase: dashboardTimUsecase,
	}
}

func (t *AdminDashboardTimHandler) MountAdmin(group *echo.Group) {
	group.GET("/dashboard-tim", t.GetDataMahasiswa)
}

// GetDataMahasiswa retrieves a list of mahasiswa that has already done pendaftaran
// @Summary Get mahasiswa pendaftaran
// @Description Retrieves a list of mahasiswa that has already done pendaftaran
// @Tags Dashboard Tim TA
// @Accept json
// @Produce json
// @Param id_dosen query string true "Dosen ID"
// @Param limit query string false "Limit"
// @Param offset query string false "Offset"
// @Success 200 {object} []entity.DataPendaftaran "List mahasiswa successfully retrieved"
// @Router /dashboard-tim [get]
func (t *AdminDashboardTimHandler) GetDataMahasiswa(c echo.Context) error {
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}

	listMahasiswa, err := t.dashboardTimUseCase.GetDataMahasiswa(offset, size)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa))

}
