package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/entity"

	"gorm.io/gorm"
)

type JadwalSeminarRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewJadwalSeminarRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *JadwalSeminarRepo {
	return &JadwalSeminarRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *JadwalSeminarRepo) NewJadwalSeminar(jadwal entity.JadwalSeminar) (output entity.JadwalSeminar, err error) {
	jadwal.ID = uuid.New().String()
	jadwal.CreatedAt = time.Now()
	result := repo.DBWrite.Create(&jadwal)
	if result.Error != nil {
		return entity.JadwalSeminar{}, result.Error
	}
	return jadwal, nil
}

func (repo *JadwalSeminarRepo) GetJadwalSeminarByBatch(idBatch string, offset, size int) (output []entity.JadwalSeminar, err error) {
	var jadwalSeminar []models.JadwalSeminar
	var dosbing models.Pengguna
	var dosuji models.Pengguna
	var mahasiswa models.Pengguna
	var dataJadwal []entity.JadwalSeminar
	result := repo.DBRead.Limit(size).Offset(offset).Where("id_batch = ?", idBatch).Find(&jadwalSeminar)

	for _, jadwal := range jadwalSeminar {
		res := repo.DBRead.Where("id = ?", jadwal.IdMahasiswa).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", jadwal.IdDosbing).Find(&dosbing)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", jadwal.IdDosuji).Find(&dosuji)
		if res.Error != nil {
			return nil, res.Error
		}

		data := entity.JadwalSeminar{
			ID:          jadwal.Id,
			Nim:         mahasiswa.NIM,
			Name:        mahasiswa.Nama,
			CreatedAt:   jadwal.CreatedAt,
			ScheduledAt: jadwal.ScheduledAt,
			NamaDosbing: dosbing.Nama,
			NamaDosuji1: dosuji.Nama,
			Ruang:       jadwal.Ruang,
			IDBatch:     jadwal.IdBatch,
			Judul:       jadwal.Judul,
		}

		dataJadwal = append(dataJadwal, data)

	}

	if result.Error != nil {
		return nil, err
	}
	return dataJadwal, nil
}

func (repo *JadwalSeminarRepo) UpdateJadwalSeminar(jadwalSeminar entity.JadwalSeminar) (entity.JadwalSeminar, error) {
	err := repo.DBWrite.Save(&jadwalSeminar).Error
	if err != nil {
		return jadwalSeminar, err
	}
	return jadwalSeminar, nil
}

func (repo *JadwalSeminarRepo) DeleteJadwalSeminar(id string) (entity.JadwalSeminar, error) {
	jadwalSeminar := entity.JadwalSeminar{ID: id}
	err := repo.DBWrite.Delete(&jadwalSeminar).Error

	if err != nil {
		return jadwalSeminar, err
	}

	return jadwalSeminar, err
}

func (repo *JadwalSeminarRepo) GetJadwalSeminarAll() (output []entity.JadwalSeminar, err error) {
	var jadwalSeminar []entity.JadwalSeminar
	result := repo.DBRead.Find(&jadwalSeminar)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSeminar, nil
}

func (repo *JadwalSeminarRepo) GetJadwalSeminarInDateRange(startDate, endDate string) ([]entity.JadwalSeminar, error) {
	var jadwalSeminar []entity.JadwalSeminar
	result := repo.DBRead.Where("created_at BETWEEN ? AND ?", startDate, endDate).Find(&jadwalSeminar)
	if result.Error != nil {
		return nil, result.Error
	}
	return jadwalSeminar, nil
}
