package usecase

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/entity"
)

type JadwalSeminarUsecase interface {
	AddJadwalSeminar(jadwalSeminar entity.JadwalSeminar) (entity.JadwalSeminar, error)
	GetJadwalSeminarByBatch(idBatch string, offset, size int) ([]entity.JadwalSeminar, error)
	EditJadwalSeminar(editInput entity.JadwalSeminar) (entity.JadwalSeminar, error)
	DeleteJadwalSeminar(id string) (entity.JadwalSeminar, error)
	GetJadwalSeminarAll() ([]entity.JadwalSeminar, error)
	GetJadwalSeminarInDateRange(startDate string, endDate string) ([]entity.JadwalSeminar, error)
}

type JadwalSeminarUc struct {
	jadwalSeminarRepo JadwalSeminarRepository
}

func NewJadwalSeminarUseCase(jadwalSeminarRepo JadwalSeminarRepository) *JadwalSeminarUc {
	return &JadwalSeminarUc{
		jadwalSeminarRepo: jadwalSeminarRepo,
	}
}

func (uc *JadwalSeminarUc) AddJadwalSeminar(batchseminar entity.JadwalSeminar) (entity.JadwalSeminar, error) {
	newJadwalSeminar, err := uc.jadwalSeminarRepo.NewJadwalSeminar(batchseminar)

	if err != nil {
		return newJadwalSeminar, err
	}

	return newJadwalSeminar, nil

}

func (uc *JadwalSeminarUc) GetJadwalSeminarByBatch(idBatch string, offset, size int) ([]entity.JadwalSeminar, error) {
	return uc.jadwalSeminarRepo.GetJadwalSeminarByBatch(idBatch, offset, size)
}

func (uc *JadwalSeminarUc) EditJadwalSeminar(editInput entity.JadwalSeminar) (entity.JadwalSeminar, error) {
	return uc.jadwalSeminarRepo.UpdateJadwalSeminar(editInput)
}

func (uc *JadwalSeminarUc) DeleteJadwalSeminar(id string) (entity.JadwalSeminar, error) {
	return uc.jadwalSeminarRepo.DeleteJadwalSeminar(id)
}

func (uc *JadwalSeminarUc) GetJadwalSeminarAll() ([]entity.JadwalSeminar, error) {
	return uc.jadwalSeminarRepo.GetJadwalSeminarAll()
}

func (uc *JadwalSeminarUc) GetJadwalSeminarInDateRange(startDate string, endDate string) ([]entity.JadwalSeminar, error) {
	return uc.jadwalSeminarRepo.GetJadwalSeminarInDateRange(startDate, endDate)
}
