package repository

import (
	// "time"

	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/entity"
	"gorm.io/gorm"
)

type JadwalSidangRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewJadwalSidangRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *JadwalSidangRepo {
	return &JadwalSidangRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *JadwalSidangRepo) NewJadwalSidang(jadwal entity.JadwalSidang) (output entity.JadwalSidang, err error) {
	jadwal.ID = uuid.New().String()
	jadwal.CreatedAt = time.Now()

	result := repo.DBWrite.Create(&jadwal)
	if result.Error != nil {
		return entity.JadwalSidang{}, result.Error
	}
	return jadwal, nil
}

func (repo *JadwalSidangRepo) GetJadwalSidangByBatch(idBatch string, offset, size int) (output []entity.JadwalSidang, err error) {
	var jadwalSidang []models.JadwalSidang
	var dosbing models.Pengguna
	var dosuji models.Pengguna
	var dosuji2 models.Pengguna
	var mahasiswa models.Pengguna
	var dataJadwal []entity.JadwalSidang
	result := repo.DBRead.Limit(size).Offset(offset).Where("id_batch = ?", idBatch).Find(&jadwalSidang)

	for _, jadwal := range jadwalSidang {
		res := repo.DBRead.Where("id = ?", jadwal.IdMahasiswa).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", jadwal.IdDosbing).Find(&dosbing)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", jadwal.IdDosuji1).Find(&dosuji)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", jadwal.IdDosuji2).Find(&dosuji2)
		if res.Error != nil {
			return nil, res.Error
		}

		data := entity.JadwalSidang{
			ID:          jadwal.Id,
			Nim:         mahasiswa.NIM,
			Name:        mahasiswa.Nama,
			CreatedAt:   jadwal.CreatedAt,
			ScheduledAt: jadwal.ScheduledAt,
			NamaDosbing: dosbing.Nama,
			NamaDosuji1: dosuji.Nama,
			NamaDosuji2: dosuji2.Nama,
			Ruang:       jadwal.Ruang,
			IDBatch:     jadwal.IdBatch,
			Judul:       jadwal.Judul,
		}

		dataJadwal = append(dataJadwal, data)

	}

	if result.Error != nil {
		return nil, err
	}
	return dataJadwal, nil
}

func (repo *JadwalSidangRepo) GetJadwalSidangAll() (output []entity.JadwalSidang, err error) {
	var jadwalSidang []entity.JadwalSidang
	result := repo.DBRead.Find(&jadwalSidang)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSidang, nil
}
func (repo *JadwalSidangRepo) UpdateJadwalSidang(jadwalSidang entity.JadwalSidang) (entity.JadwalSidang, error) {
	err := repo.DBWrite.Save(&jadwalSidang).Error
	if err != nil {
		return jadwalSidang, err
	}
	return jadwalSidang, nil
}

func (repo *JadwalSidangRepo) DeleteJadwalSidang(id string) (entity.JadwalSidang, error) {
	jadwalSidang := entity.JadwalSidang{ID: id}
	err := repo.DBWrite.Delete(&jadwalSidang).Error

	if err != nil {
		return jadwalSidang, err
	}

	return jadwalSidang, err
}

func (repo *JadwalSidangRepo) GetJadwalSidangInDateRange(startDate, endDate string) ([]entity.JadwalSidang, error) {
	var jadwalSidang []entity.JadwalSidang
	result := repo.DBRead.Where("created_at BETWEEN ? AND ?", startDate, endDate).Find(&jadwalSidang)
	if result.Error != nil {
		return nil, result.Error
	}
	return jadwalSidang, nil
}
