package usecase

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/entity"
)

type JadwalSidangUsecase interface {
	AddJadwalSidang(jadwalSidang entity.JadwalSidang) (entity.JadwalSidang, error)
	GetJadwalSidangByBatch(idBatch string, offset, size int) ([]entity.JadwalSidang, error)
	EditJadwalSidang(editInput entity.JadwalSidang) (entity.JadwalSidang, error)
	DeleteJadwalSidang(id string) (entity.JadwalSidang, error)
	GetJadwalSidangAll() ([]entity.JadwalSidang, error)
	GetJadwalSidangInDateRange(startDate string, endDate string) ([]entity.JadwalSidang, error)
}

type JadwalSidangUc struct {
	jadwalSidangRepo JadwalSidangRepository
}

func NewJadwalSidangUseCase(jadwalSidangRepo JadwalSidangRepository) *JadwalSidangUc {
	return &JadwalSidangUc{
		jadwalSidangRepo: jadwalSidangRepo,
	}
}

func (uc *JadwalSidangUc) AddJadwalSidang(batchSidang entity.JadwalSidang) (entity.JadwalSidang, error) {
	newJadwalSidang, err := uc.jadwalSidangRepo.NewJadwalSidang(batchSidang)

	if err != nil {
		return newJadwalSidang, err
	}

	return newJadwalSidang, nil

}

func (uc *JadwalSidangUc) GetJadwalSidangByBatch(idBatch string, offset, size int) ([]entity.JadwalSidang, error) {
	return uc.jadwalSidangRepo.GetJadwalSidangByBatch(idBatch, offset, size)
}

func (uc *JadwalSidangUc) EditJadwalSidang(editInput entity.JadwalSidang) (entity.JadwalSidang, error) {
	return uc.jadwalSidangRepo.UpdateJadwalSidang(editInput)
}

func (uc *JadwalSidangUc) DeleteJadwalSidang(id string) (entity.JadwalSidang, error) {
	return uc.jadwalSidangRepo.DeleteJadwalSidang(id)
}

func (uc *JadwalSidangUc) GetJadwalSidangAll() ([]entity.JadwalSidang, error) {
	return uc.jadwalSidangRepo.GetJadwalSidangAll()
}

func (uc *JadwalSidangUc) GetJadwalSidangInDateRange(startDate string, endDate string) ([]entity.JadwalSidang, error) {
	return uc.jadwalSidangRepo.GetJadwalSidangInDateRange(startDate, endDate)
}
