package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pengguna/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pengguna/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pengguna/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminPenggunaHandler struct {
	penggunaUsecase usecase.PenggunaUsecase
}

func NewAdminPenggunaHandler(cfg config.PenggunaTransportConfig) *AdminPenggunaHandler {
	penggunaRepository := repository.NewPenggunaRepository(cfg.DBWrite, cfg.DBRead)
	penggunaUsecase := usecase.NewPenggunaUseCase(penggunaRepository)

	return &AdminPenggunaHandler{
		penggunaUsecase: penggunaUsecase,
	}

}

func (t *AdminPenggunaHandler) MountAdmin(group *echo.Group) {
	group.GET("/pengguna", t.GetPenggunaByNim)
}

func (t *AdminPenggunaHandler) GetPenggunaByNim(c echo.Context) error {
	idMahasiswa := c.QueryParam("id_mahasiswa")

	pengguna, err := t.penggunaUsecase.GetPenggunaByNIM(idMahasiswa)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pengguna successfully retreived", pengguna))
}
