package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminSystemLogHandler struct {
	systemLogUsecase usecase.SystemLogUsecase
}

func NewAdminSystemLogHandler(cfg config.SystemLogTransportConfig) *AdminSystemLogHandler {
	systemLogRepository := repository.NewSystemLogRepository(cfg.DBWrite, cfg.DBRead)
	systemLogUsecase := usecase.NewSystemLogUseCase(systemLogRepository)

	return &AdminSystemLogHandler{
		systemLogUsecase: systemLogUsecase,
	}

}

func (t *AdminSystemLogHandler) MountAdmin(group *echo.Group) {
	group.GET("/system-logs", t.GetSystemLog)
}

// GetSystemLog retrieves a list of system logs.
// @Summary Retrieve system logs
// @Description Retrieves a list of system logs.
// @Tags System Logs
// @Produce json
// @Param limit query int false "Limit number of logs per page"
// @Param offset query int false "Offset for pagination"
// @Success 200 {object} []entity.SystemLog
// @Router /system-logs [get]
func (t *AdminSystemLogHandler) GetSystemLog(c echo.Context) error {
	param, err := utils.GetLimitOffset(c.QueryParams())
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	listGroupLimit, err := t.systemLogUsecase.GetSystemLog(param)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "System logs successfully retreived", listGroupLimit))
}
