package usecase

import (
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"os"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TopicUsecase interface {
	AddTopic(topic []entity.Topik) ([]entity.Topik, error)
	GetTopic(param utils.PageLimitSearchID) ([]entity.Topik, error)
	GetTopicByID(param string) (entity.Topik, error)
	UpdateTopic(topic entity.Topik) (entity.Topik, error)
	DeleteTopic(param string) (error)
}

type TopicUc struct {
	topicAllocationRepo TopicAllocationRepo
}

func NewTopicUc(topicAllocationRepo TopicAllocationRepo) *TopicUc {
	return &TopicUc{
		topicAllocationRepo: topicAllocationRepo,
	}
}

func (uc *TopicUc) AddTopic(topics []entity.Topik) ([]entity.Topik, error) {
	url := os.Getenv("S1_URL")
	res, reqErr := http.Get(url + "/api/internal/batch-seminar-live")
	if reqErr != nil {
		return nil, reqErr
	}
	defer res.Body.Close()

	body, parseErr := io.ReadAll(res.Body)
	if parseErr != nil {
		return nil, parseErr
	}

	var parsedRes map[string]interface{}
	parsedErr := json.Unmarshal(body, &parsedRes)
	if parsedErr != nil {
		return nil, parsedErr
	}

	for _, topic := range topics {
		_, err := uc.topicAllocationRepo.NewTopic(topic)
		if err != nil {
			topic.Periode = fmt.Sprint(parsedRes["semester"]) + "-" + fmt.Sprint(parsedRes["year"])
			_, err := uc.topicAllocationRepo.NewTopic(topic)
			if err != nil {
				return nil, err
			}
		}
	}

	return topics, nil
}

func (uc *TopicUc) GetTopic(param utils.PageLimitSearchID) ([]entity.Topik, error) {
	
	url := os.Getenv("S1_URL")
	res, reqErr := http.Get(url + "/api/internal/batch-seminar-live")
	if reqErr != nil {
		return nil, reqErr
	}
	defer res.Body.Close()

	body, parseErr := io.ReadAll(res.Body)
	if parseErr != nil {
		return nil, parseErr
	}

	var parsedRes map[string]interface{}
	parsedErr := json.Unmarshal(body, &parsedRes)
	if parsedErr != nil {
		return nil, parsedErr
	}

	param.Periode = fmt.Sprint(parsedRes["semester"]) + "-" + fmt.Sprint(parsedRes["year"])

	return uc.topicAllocationRepo.GetTopic(param.Page, param.Limit, param.Search, param.ID, param.Periode)
}

func (uc *TopicUc) GetTopicByID(param string) (entity.Topik, error) {
	return uc.topicAllocationRepo.GetTopicByID(param)
}

func (uc *TopicUc) UpdateTopic(topic entity.Topik) (entity.Topik, error) {
	updatedTopic, err := uc.topicAllocationRepo.UpdateTopic(topic)

	if err != nil {
		return entity.Topik{}, err
	}

	return updatedTopic, nil
}

func (uc *TopicUc) DeleteTopic(param string) (error) {
	err := uc.topicAllocationRepo.DeleteTopic(param)

	if err != nil {
		return err
	}

	return nil
}