package utils

import (
	"net/url"
	"strconv"
)

type LimitOffset struct {
	Limit  int
	Offset int
}

type ResponseDetail struct {
	Success bool        `json:"success"`
	Code    int         `json:"code"`
	Message string      `json:"message"`
	Data    interface{} `json:"data,omitempty"`
}

type PageLimitSearchID struct {
	Page int
	Limit int
	Search string
	ID string
	Periode string
}

func ResponseDetailOutput(success bool, code int, message string, data interface{}) ResponseDetail {
	res := ResponseDetail{
		Success: success,
		Code:    code,
		Message: message,
		Data:    data,
	}
	return res
}

func GetLimitOffset(urlValues url.Values) (output LimitOffset, err error) {
	param := new(LimitOffset)

	offset := urlValues.Get("offset")
	if offset != "" {
		offsetInt, err := strconv.Atoi(offset)
		if err != nil {
			return LimitOffset{}, err
		}
		param.Offset = offsetInt
	} else {
		param.Offset = 0
	}
	limit := urlValues.Get("limit")
	if limit != "" {
		limitInt, err := strconv.Atoi(limit)
		if err != nil {
			return LimitOffset{}, err
		}
		param.Limit = limitInt
	} else {
		param.Limit = 20
	}

	return *param, nil
}

func GetID(urlValues url.Values) (output string, err error) {
	id := urlValues.Get("id")
	if id != "" {
		return id, nil
	} else {
		return "", nil
	}
}

func GetPageLimitSearchID(urlValues url.Values) (output PageLimitSearchID, err error) {
	param := new(PageLimitSearchID)
	page := urlValues.Get("page")
	limit := urlValues.Get("limit")
	search := urlValues.Get("search")
	id := urlValues.Get("idPembimbing")

	if page != "" {
		pageInt, err := strconv.Atoi(page)

		if err != nil {
			return PageLimitSearchID{}, err
		}

		param.Page = pageInt
	}

	if limit != "" {
		limitInt, err := strconv.Atoi(limit)

		if err != nil {
			return PageLimitSearchID{}, err
		}

		param.Limit = limitInt
	}

	if search != "" {
		param.Search = search
	}

	if id != "" {
		param.ID = id
	}

	return *param, nil
}